from church import (
    Address, Personal, Text,
    Network, Datetime, File, Science,
    Development, Food, Hardware
)

__version__ = '0.0.1'
__author__ = 'Lk Geimfari'


class Church(object):
    """Basic class"""

    def __init__(self, app=None):
        """
        Initialize the flask extension
        :param app: Flask application instance
        :return: None
        """
        if app is not None:
            self.init_app(app)

    def init_app(self, app):
        """
        Initializes the Flask application with this extension. It grabs
        the necessary configuration values from `app.config`.
        :param app: Flask application object
        """
        if not hasattr(app, 'extensions'):
            app.extensions = {}

        app.extensions['church'] = self
        app.context_processor(lambda: {
            'church': self
        })

        locale = app.config.get('CHURCH_LOCALE', 'EN_US')

        # Initialize classes.
        self.personal = Personal(locale)
        self.address = Address(locale)
        self.text = Text(locale)
        self.science = Science(locale)
        self.datetime = Datetime(locale)
        self.food = Food(locale)

        # Classes without locales.
        self.hardware = Hardware()
        self.development = Development()
        self.network = Network()
        self.file = File()

    def __repr__(self):
        return '<Flask-Church v{0}>'.format(__version__)

    def churchify(self, attributes=None, as_objects=False):
        """
        Function for generate dictionary with fake data.
        :param attributes: Sequence (tuple, list) of attributes
        :param as_objects: If True then return callable object
        else return result.
        :return: Dictionary of fake data.
        Example (as_objects=True):
        {
        'username': <function Personal.username at 0x7f8f68f6f598>,
        'email': <function Personal.email at 0x7f8f68f6f7b8>,
        'password': <function Personal.password at 0x7f8f68f6f730>
        }

        """

        if attributes is None:
            raise ValueError('`attributes` cannot be empty')

        data = {}
        for attr in attributes:
            if hasattr(self.personal, attr):
                data[attr] = getattr(self.personal, attr)() \
                    if not as_objects else getattr(self.personal, attr)
        return data
